"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.detectTestingLibraryUtils = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const node_utils_1 = require("./node-utils");
const utils_1 = require("./utils");
const DEFAULT_FILENAME_PATTERN = '^.*\\.(test|spec)\\.[jt]sx?$';
const FIRE_EVENT_NAME = 'fireEvent';
const RENDER_NAME = 'render';
function detectTestingLibraryUtils(ruleCreate) {
    return (context, optionsWithDefault) => {
        var _a;
        let importedTestingLibraryNode = null;
        let importedCustomModuleNode = null;
        const customModule = context.settings['testing-library/utils-module'];
        const filenamePattern = (_a = context.settings['testing-library/filename-pattern']) !== null && _a !== void 0 ? _a : DEFAULT_FILENAME_PATTERN;
        const customRenders = context.settings['testing-library/custom-renders'];
        function isTestingLibraryUtil(node, isUtilCallback) {
            const referenceNode = node_utils_1.getReferenceNode(node);
            const referenceNodeIdentifier = node_utils_1.getPropertyIdentifierNode(referenceNode);
            const importedUtilSpecifier = getImportedUtilSpecifier(referenceNodeIdentifier);
            const originalNodeName = node_utils_1.isImportSpecifier(importedUtilSpecifier) &&
                importedUtilSpecifier.local.name !== importedUtilSpecifier.imported.name
                ? importedUtilSpecifier.imported.name
                : undefined;
            if (!isUtilCallback(node.name, originalNodeName)) {
                return false;
            }
            if (isAggressiveModuleReportingEnabled()) {
                return true;
            }
            return isNodeComingFromTestingLibrary(referenceNodeIdentifier);
        }
        const isAggressiveModuleReportingEnabled = () => !customModule;
        const isAggressiveRenderReportingEnabled = () => !Array.isArray(customRenders) || customRenders.length === 0;
        const getTestingLibraryImportNode = () => {
            return importedTestingLibraryNode;
        };
        const getCustomModuleImportNode = () => {
            return importedCustomModuleNode;
        };
        const getTestingLibraryImportName = () => {
            return node_utils_1.getImportModuleName(importedTestingLibraryNode);
        };
        const getCustomModuleImportName = () => {
            return node_utils_1.getImportModuleName(importedCustomModuleNode);
        };
        const isTestingLibraryImported = () => {
            return (isAggressiveModuleReportingEnabled() ||
                !!importedTestingLibraryNode ||
                !!importedCustomModuleNode);
        };
        const isValidFilename = () => {
            const fileName = context.getFilename();
            return !!fileName.match(filenamePattern);
        };
        const isGetQueryVariant = (node) => {
            return /^get(All)?By.+$/.test(node.name);
        };
        const isQueryQueryVariant = (node) => {
            return /^query(All)?By.+$/.test(node.name);
        };
        const isFindQueryVariant = (node) => {
            return /^find(All)?By.+$/.test(node.name);
        };
        const isSyncQuery = (node) => {
            return isGetQueryVariant(node) || isQueryQueryVariant(node);
        };
        const isAsyncQuery = (node) => {
            return isFindQueryVariant(node);
        };
        const isQuery = (node) => {
            return isSyncQuery(node) || isAsyncQuery(node);
        };
        const isCustomQuery = (node) => {
            return isQuery(node) && !utils_1.ALL_QUERIES_COMBINATIONS.includes(node.name);
        };
        const isAsyncUtil = (node, validNames = utils_1.ASYNC_UTILS) => {
            return isTestingLibraryUtil(node, (identifierNodeName, originalNodeName) => {
                return (validNames.includes(identifierNodeName) ||
                    validNames.includes(originalNodeName));
            });
        };
        const isFireEventMethod = (node) => {
            const fireEventUtil = findImportedUtilSpecifier(FIRE_EVENT_NAME);
            let fireEventUtilName;
            if (fireEventUtil) {
                fireEventUtilName = experimental_utils_1.ASTUtils.isIdentifier(fireEventUtil)
                    ? fireEventUtil.name
                    : fireEventUtil.local.name;
            }
            else if (isAggressiveModuleReportingEnabled()) {
                fireEventUtilName = FIRE_EVENT_NAME;
            }
            if (!fireEventUtilName) {
                return false;
            }
            const parentMemberExpression = node_utils_1.isMemberExpression(node.parent) ? node.parent : undefined;
            if (!parentMemberExpression) {
                return false;
            }
            if ([fireEventUtilName, FIRE_EVENT_NAME].includes(node.name) ||
                (experimental_utils_1.ASTUtils.isIdentifier(parentMemberExpression.object) &&
                    parentMemberExpression.object.name === node.name)) {
                return false;
            }
            const regularCall = experimental_utils_1.ASTUtils.isIdentifier(parentMemberExpression.object) &&
                parentMemberExpression.object.name === fireEventUtilName;
            const wildcardCall = node_utils_1.isMemberExpression(parentMemberExpression.object) &&
                experimental_utils_1.ASTUtils.isIdentifier(parentMemberExpression.object.object) &&
                parentMemberExpression.object.object.name === fireEventUtilName &&
                experimental_utils_1.ASTUtils.isIdentifier(parentMemberExpression.object.property) &&
                parentMemberExpression.object.property.name === FIRE_EVENT_NAME;
            return regularCall || wildcardCall;
        };
        const isRenderUtil = (node) => {
            return isTestingLibraryUtil(node, (identifierNodeName, originalNodeName) => {
                if (isAggressiveRenderReportingEnabled()) {
                    return identifierNodeName.toLowerCase().includes(RENDER_NAME);
                }
                return [RENDER_NAME, ...customRenders].some((validRenderName) => {
                    let isMatch = false;
                    if (validRenderName === identifierNodeName) {
                        isMatch = true;
                    }
                    if (!!originalNodeName && validRenderName === originalNodeName) {
                        isMatch = true;
                    }
                    return isMatch;
                });
            });
        };
        const isPresenceAssert = (node) => {
            const { matcher, isNegated } = node_utils_1.getAssertNodeInfo(node);
            if (!matcher) {
                return false;
            }
            return isNegated
                ? utils_1.ABSENCE_MATCHERS.includes(matcher)
                : utils_1.PRESENCE_MATCHERS.includes(matcher);
        };
        const isAbsenceAssert = (node) => {
            const { matcher, isNegated } = node_utils_1.getAssertNodeInfo(node);
            if (!matcher) {
                return false;
            }
            return isNegated
                ? utils_1.PRESENCE_MATCHERS.includes(matcher)
                : utils_1.ABSENCE_MATCHERS.includes(matcher);
        };
        const findImportedUtilSpecifier = (specifierName) => {
            var _a;
            const node = (_a = getCustomModuleImportNode()) !== null && _a !== void 0 ? _a : getTestingLibraryImportNode();
            if (!node) {
                return null;
            }
            if (node_utils_1.isImportDeclaration(node)) {
                const namedExport = node.specifiers.find((n) => {
                    return (node_utils_1.isImportSpecifier(n) &&
                        [n.imported.name, n.local.name].includes(specifierName));
                });
                if (namedExport) {
                    return namedExport;
                }
                return node.specifiers.find((n) => node_utils_1.isImportNamespaceSpecifier(n));
            }
            else {
                const requireNode = node.parent;
                if (experimental_utils_1.ASTUtils.isIdentifier(requireNode.id)) {
                    return requireNode.id;
                }
                const destructuring = requireNode.id;
                const property = destructuring.properties.find((n) => node_utils_1.isProperty(n) &&
                    experimental_utils_1.ASTUtils.isIdentifier(n.key) &&
                    n.key.name === specifierName);
                if (!property) {
                    return undefined;
                }
                return property.key;
            }
        };
        const getImportedUtilSpecifier = (node) => {
            const identifierName = node_utils_1.getPropertyIdentifierNode(node)
                .name;
            return findImportedUtilSpecifier(identifierName);
        };
        const canReportErrors = () => {
            return isTestingLibraryImported() && isValidFilename();
        };
        const isNodeComingFromTestingLibrary = (node) => {
            const importNode = getImportedUtilSpecifier(node);
            if (!importNode) {
                return false;
            }
            const identifierName = node_utils_1.getPropertyIdentifierNode(node)
                .name;
            return node_utils_1.hasImportMatch(importNode, identifierName);
        };
        const helpers = {
            getTestingLibraryImportNode,
            getCustomModuleImportNode,
            getTestingLibraryImportName,
            getCustomModuleImportName,
            isTestingLibraryImported,
            isValidFilename,
            isGetQueryVariant,
            isQueryQueryVariant,
            isFindQueryVariant,
            isSyncQuery,
            isAsyncQuery,
            isQuery,
            isCustomQuery,
            isAsyncUtil,
            isFireEventMethod,
            isRenderUtil,
            isPresenceAssert,
            isAbsenceAssert,
            canReportErrors,
            findImportedUtilSpecifier,
            isNodeComingFromTestingLibrary,
        };
        const detectionInstructions = {
            ImportDeclaration(node) {
                if (!importedTestingLibraryNode &&
                    /testing-library/g.test(node.source.value)) {
                    importedTestingLibraryNode = node;
                }
                if (!importedCustomModuleNode &&
                    String(node.source.value).endsWith(customModule)) {
                    importedCustomModuleNode = node;
                }
            },
            [`CallExpression > Identifier[name="require"]`](node) {
                const callExpression = node.parent;
                const { arguments: args } = callExpression;
                if (!importedTestingLibraryNode &&
                    args.some((arg) => node_utils_1.isLiteral(arg) &&
                        typeof arg.value === 'string' &&
                        /testing-library/g.test(arg.value))) {
                    importedTestingLibraryNode = callExpression;
                }
                if (!importedCustomModuleNode &&
                    args.some((arg) => node_utils_1.isLiteral(arg) &&
                        typeof arg.value === 'string' &&
                        arg.value.endsWith(customModule))) {
                    importedCustomModuleNode = callExpression;
                }
            },
        };
        const ruleInstructions = ruleCreate(context, optionsWithDefault, helpers);
        const enhancedRuleInstructions = {};
        const allKeys = new Set(Object.keys(detectionInstructions).concat(Object.keys(ruleInstructions)));
        allKeys.forEach((instruction) => {
            enhancedRuleInstructions[instruction] = (node) => {
                if (instruction in detectionInstructions) {
                    detectionInstructions[instruction](node);
                }
                if (canReportErrors() && ruleInstructions[instruction]) {
                    return ruleInstructions[instruction](node);
                }
            };
        });
        return enhancedRuleInstructions;
    };
}
exports.detectTestingLibraryUtils = detectTestingLibraryUtils;
