"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const utils_1 = require("../utils");
const node_utils_1 = require("../node-utils");
exports.RULE_NAME = 'no-container';
exports.default = experimental_utils_1.ESLintUtils.RuleCreator(utils_1.getDocsUrl)({
    name: exports.RULE_NAME,
    meta: {
        type: 'problem',
        docs: {
            description: 'Disallow the use of container methods',
            category: 'Best Practices',
            recommended: 'error',
        },
        messages: {
            noContainer: 'Avoid using container methods. Prefer using the methods from Testing Library, such as "getByRole()"',
        },
        fixable: null,
        schema: [
            {
                type: 'object',
                properties: {
                    renderFunctions: {
                        type: 'array',
                    },
                },
            },
        ],
    },
    defaultOptions: [
        {
            renderFunctions: [],
        },
    ],
    create(context, [options]) {
        const { renderFunctions } = options;
        const destructuredContainerPropNames = [];
        let renderWrapperName = null;
        let containerName = null;
        let containerCallsMethod = false;
        function showErrorIfChainedContainerMethod(innerNode) {
            if (node_utils_1.isMemberExpression(innerNode)) {
                if (experimental_utils_1.ASTUtils.isIdentifier(innerNode.object)) {
                    const isContainerName = innerNode.object.name === containerName;
                    const isRenderWrapper = innerNode.object.name === renderWrapperName;
                    containerCallsMethod =
                        experimental_utils_1.ASTUtils.isIdentifier(innerNode.property) &&
                            innerNode.property.name === 'container' &&
                            isRenderWrapper;
                    if (isContainerName || containerCallsMethod) {
                        context.report({
                            node: innerNode,
                            messageId: 'noContainer',
                        });
                    }
                }
                showErrorIfChainedContainerMethod(innerNode.object);
            }
        }
        return {
            VariableDeclarator(node) {
                if (node_utils_1.isRenderVariableDeclarator(node, ['render', ...renderFunctions])) {
                    if (node_utils_1.isObjectPattern(node.id)) {
                        const containerIndex = node.id.properties.findIndex((property) => node_utils_1.isProperty(property) &&
                            experimental_utils_1.ASTUtils.isIdentifier(property.key) &&
                            property.key.name === 'container');
                        const nodeValue = containerIndex !== -1 && node.id.properties[containerIndex].value;
                        if (experimental_utils_1.ASTUtils.isIdentifier(nodeValue)) {
                            containerName = nodeValue.name;
                        }
                        else {
                            node_utils_1.isObjectPattern(nodeValue) &&
                                nodeValue.properties.forEach((property) => node_utils_1.isProperty(property) &&
                                    experimental_utils_1.ASTUtils.isIdentifier(property.key) &&
                                    destructuredContainerPropNames.push(property.key.name));
                        }
                    }
                    else {
                        renderWrapperName = experimental_utils_1.ASTUtils.isIdentifier(node.id) && node.id.name;
                    }
                }
            },
            CallExpression(node) {
                if (node_utils_1.isMemberExpression(node.callee)) {
                    showErrorIfChainedContainerMethod(node.callee);
                }
                else {
                    experimental_utils_1.ASTUtils.isIdentifier(node.callee) &&
                        destructuredContainerPropNames.includes(node.callee.name) &&
                        context.report({
                            node,
                            messageId: 'noContainer',
                        });
                }
            },
        };
    },
});
