"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const utils_1 = require("../utils");
const node_utils_1 = require("../node-utils");
exports.RULE_NAME = 'no-debug';
exports.default = experimental_utils_1.ESLintUtils.RuleCreator(utils_1.getDocsUrl)({
    name: exports.RULE_NAME,
    meta: {
        type: 'problem',
        docs: {
            description: 'Disallow unnecessary debug usages in the tests',
            category: 'Best Practices',
            recommended: 'warn',
        },
        messages: {
            noDebug: 'Unexpected debug statement',
        },
        fixable: null,
        schema: [
            {
                type: 'object',
                properties: {
                    renderFunctions: {
                        type: 'array',
                    },
                },
            },
        ],
    },
    defaultOptions: [
        {
            renderFunctions: [],
        },
    ],
    create(context, [options]) {
        let hasDestructuredDebugStatement = false;
        const renderVariableDeclarators = [];
        const { renderFunctions } = options;
        let hasImportedScreen = false;
        let wildcardImportName = null;
        return {
            VariableDeclarator(node) {
                if (node_utils_1.isRenderVariableDeclarator(node, ['render', ...renderFunctions])) {
                    if (node_utils_1.isObjectPattern(node.id) &&
                        node.id.properties.some((property) => node_utils_1.isProperty(property) &&
                            experimental_utils_1.ASTUtils.isIdentifier(property.key) &&
                            property.key.name === 'debug')) {
                        hasDestructuredDebugStatement = true;
                    }
                    if (node.id.type === 'Identifier') {
                        renderVariableDeclarators.push(node);
                    }
                }
            },
            [`VariableDeclarator > CallExpression > Identifier[name="require"]`](node) {
                const { arguments: args } = node.parent;
                const literalNodeScreenModuleName = args.find((args) => node_utils_1.isLiteral(args) &&
                    typeof args.value === 'string' &&
                    utils_1.LIBRARY_MODULES.includes(args.value));
                if (!literalNodeScreenModuleName) {
                    return;
                }
                const declaratorNode = node.parent
                    .parent;
                hasImportedScreen =
                    node_utils_1.isObjectPattern(declaratorNode.id) &&
                        declaratorNode.id.properties.some((property) => node_utils_1.isProperty(property) &&
                            experimental_utils_1.ASTUtils.isIdentifier(property.key) &&
                            property.key.name === 'screen');
            },
            ImportDeclaration(node) {
                if (!utils_1.hasTestingLibraryImportModule(node)) {
                    return;
                }
                hasImportedScreen = node.specifiers.some((s) => node_utils_1.isImportSpecifier(s) && s.imported.name === 'screen');
            },
            'ImportDeclaration ImportNamespaceSpecifier'(node) {
                const importDeclarationNode = node.parent;
                if (!utils_1.hasTestingLibraryImportModule(importDeclarationNode)) {
                    return;
                }
                wildcardImportName = node.local && node.local.name;
            },
            [`CallExpression > Identifier[name="debug"]`](node) {
                if (hasDestructuredDebugStatement) {
                    context.report({
                        node,
                        messageId: 'noDebug',
                    });
                }
            },
            [`CallExpression > MemberExpression > Identifier[name="debug"]`](node) {
                const memberExpression = node.parent;
                const identifier = memberExpression.object;
                const memberExpressionName = identifier.name;
                const isScreenDebugUsed = hasImportedScreen && memberExpressionName === 'screen';
                const isNamespaceDebugUsed = wildcardImportName && memberExpressionName === wildcardImportName;
                if (isScreenDebugUsed || isNamespaceDebugUsed) {
                    context.report({
                        node,
                        messageId: 'noDebug',
                    });
                }
            },
            'Program:exit'() {
                renderVariableDeclarators.forEach((renderVar) => {
                    const renderVarReferences = context
                        .getDeclaredVariables(renderVar)[0]
                        .references.slice(1);
                    renderVarReferences.forEach((ref) => {
                        const parent = ref.identifier.parent;
                        if (node_utils_1.isMemberExpression(parent) &&
                            experimental_utils_1.ASTUtils.isIdentifier(parent.property) &&
                            parent.property.name === 'debug' &&
                            node_utils_1.isCallExpression(parent.parent)) {
                            context.report({
                                node: parent.property,
                                messageId: 'noDebug',
                            });
                        }
                    });
                });
            },
        };
    },
});
