"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const node_utils_1 = require("../node-utils");
const create_testing_library_rule_1 = require("../create-testing-library-rule");
exports.RULE_NAME = 'no-manual-cleanup';
const CLEANUP_LIBRARY_REGEXP = /(@testing-library\/(preact|react|svelte|vue))|@marko\/testing-library/;
exports.default = create_testing_library_rule_1.createTestingLibraryRule({
    name: exports.RULE_NAME,
    meta: {
        type: 'problem',
        docs: {
            description: 'Disallow the use of `cleanup`',
            category: 'Best Practices',
            recommended: false,
        },
        messages: {
            noManualCleanup: "`cleanup` is performed automatically by your test runner, you don't need manual cleanups.",
        },
        fixable: null,
        schema: [],
    },
    defaultOptions: [],
    create(context, _, helpers) {
        function reportImportReferences(references) {
            references.forEach((reference) => {
                const utilsUsage = reference.identifier.parent;
                if (node_utils_1.isMemberExpression(utilsUsage) &&
                    experimental_utils_1.ASTUtils.isIdentifier(utilsUsage.property) &&
                    utilsUsage.property.name === 'cleanup') {
                    context.report({
                        node: utilsUsage.property,
                        messageId: 'noManualCleanup',
                    });
                }
            });
        }
        function reportCandidateModule(moduleNode) {
            if (node_utils_1.isImportDeclaration(moduleNode)) {
                if (node_utils_1.isImportDefaultSpecifier(moduleNode.specifiers[0])) {
                    const { references } = context.getDeclaredVariables(moduleNode)[0];
                    reportImportReferences(references);
                }
                const cleanupSpecifier = moduleNode.specifiers.find((specifier) => node_utils_1.isImportSpecifier(specifier) &&
                    specifier.imported.name === 'cleanup');
                if (cleanupSpecifier) {
                    context.report({
                        node: cleanupSpecifier,
                        messageId: 'noManualCleanup',
                    });
                }
            }
            else {
                const declaratorNode = moduleNode.parent;
                if (node_utils_1.isObjectPattern(declaratorNode.id)) {
                    const cleanupProperty = declaratorNode.id.properties.find((property) => node_utils_1.isProperty(property) &&
                        experimental_utils_1.ASTUtils.isIdentifier(property.key) &&
                        property.key.name === 'cleanup');
                    if (cleanupProperty) {
                        context.report({
                            node: cleanupProperty,
                            messageId: 'noManualCleanup',
                        });
                    }
                }
                else {
                    const references = node_utils_1.getVariableReferences(context, declaratorNode);
                    reportImportReferences(references);
                }
            }
        }
        return {
            'Program:exit'() {
                const testingLibraryImportName = helpers.getTestingLibraryImportName();
                const testingLibraryImportNode = helpers.getTestingLibraryImportNode();
                const customModuleImportNode = helpers.getCustomModuleImportNode();
                if (testingLibraryImportName &&
                    testingLibraryImportNode &&
                    testingLibraryImportName.match(CLEANUP_LIBRARY_REGEXP)) {
                    reportCandidateModule(testingLibraryImportNode);
                }
                if (customModuleImportNode) {
                    reportCandidateModule(customModuleImportNode);
                }
            },
        };
    },
});
