"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findClosestBeforeHook = exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const utils_1 = require("../utils");
const node_utils_1 = require("../node-utils");
exports.RULE_NAME = 'no-render-in-setup';
function findClosestBeforeHook(node, testingFrameworkSetupHooksToFilter) {
    if (node === null) {
        return null;
    }
    if (node_utils_1.isCallExpression(node) &&
        experimental_utils_1.ASTUtils.isIdentifier(node.callee) &&
        testingFrameworkSetupHooksToFilter.includes(node.callee.name)) {
        return node.callee;
    }
    return findClosestBeforeHook(node.parent, testingFrameworkSetupHooksToFilter);
}
exports.findClosestBeforeHook = findClosestBeforeHook;
exports.default = experimental_utils_1.ESLintUtils.RuleCreator(utils_1.getDocsUrl)({
    name: exports.RULE_NAME,
    meta: {
        type: 'problem',
        docs: {
            description: 'Disallow the use of `render` in setup functions',
            category: 'Best Practices',
            recommended: false,
        },
        messages: {
            noRenderInSetup: 'Move `render` out of `{{name}}` and into individual tests.',
        },
        fixable: null,
        schema: [
            {
                type: 'object',
                properties: {
                    renderFunctions: {
                        type: 'array',
                    },
                    allowTestingFrameworkSetupHook: {
                        enum: utils_1.TESTING_FRAMEWORK_SETUP_HOOKS,
                    },
                },
                anyOf: [
                    {
                        required: ['renderFunctions'],
                    },
                    {
                        required: ['allowTestingFrameworkSetupHook'],
                    },
                ],
            },
        ],
    },
    defaultOptions: [
        {
            renderFunctions: [],
            allowTestingFrameworkSetupHook: '',
        },
    ],
    create(context, [{ renderFunctions, allowTestingFrameworkSetupHook }]) {
        let renderImportedFromTestingLib = false;
        let wildcardImportName = null;
        return {
            'ImportDeclaration[source.value=/testing-library/] ImportNamespaceSpecifier'(node) {
                wildcardImportName = node.local && node.local.name;
            },
            'ImportDeclaration[source.value=/testing-library/]'(node) {
                renderImportedFromTestingLib = node.specifiers.some((specifier) => {
                    return (node_utils_1.isImportSpecifier(specifier) && specifier.local.name === 'render');
                });
            },
            [`VariableDeclarator > CallExpression > Identifier[name="require"]`](node) {
                const { arguments: callExpressionArgs, } = node.parent;
                const testingLibImport = callExpressionArgs.find((args) => node_utils_1.isLiteral(args) &&
                    typeof args.value === 'string' &&
                    RegExp(/testing-library/, 'g').test(args.value));
                if (!testingLibImport) {
                    return;
                }
                const declaratorNode = node.parent
                    .parent;
                renderImportedFromTestingLib =
                    node_utils_1.isObjectPattern(declaratorNode.id) &&
                        declaratorNode.id.properties.some((property) => node_utils_1.isProperty(property) &&
                            experimental_utils_1.ASTUtils.isIdentifier(property.key) &&
                            property.key.name === 'render');
            },
            CallExpression(node) {
                let testingFrameworkSetupHooksToFilter = utils_1.TESTING_FRAMEWORK_SETUP_HOOKS;
                if (allowTestingFrameworkSetupHook.length !== 0) {
                    testingFrameworkSetupHooksToFilter = utils_1.TESTING_FRAMEWORK_SETUP_HOOKS.filter((hook) => hook !== allowTestingFrameworkSetupHook);
                }
                const beforeHook = findClosestBeforeHook(node, testingFrameworkSetupHooksToFilter);
                const disallowedRenderFns = renderImportedFromTestingLib || wildcardImportName
                    ? ['render', ...renderFunctions]
                    : renderFunctions;
                if (node_utils_1.isRenderFunction(node, disallowedRenderFns) && beforeHook) {
                    context.report({
                        node,
                        messageId: 'noRenderInSetup',
                        data: {
                            name: beforeHook.name,
                        },
                    });
                }
            },
        };
    },
});
