"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFindByQueryVariant = exports.WAIT_METHODS = exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const node_utils_1 = require("../node-utils");
const create_testing_library_rule_1 = require("../create-testing-library-rule");
exports.RULE_NAME = 'prefer-find-by';
exports.WAIT_METHODS = ['waitFor', 'waitForElement', 'wait'];
function getFindByQueryVariant(queryMethod) {
    return queryMethod.includes('All') ? 'findAllBy' : 'findBy';
}
exports.getFindByQueryVariant = getFindByQueryVariant;
function findRenderDefinitionDeclaration(scope, query) {
    if (!scope) {
        return null;
    }
    const variable = scope.variables.find((v) => v.name === query);
    if (variable) {
        return variable.defs
            .map(({ name }) => name)
            .filter(experimental_utils_1.ASTUtils.isIdentifier)
            .find(({ name }) => name === query);
    }
    return findRenderDefinitionDeclaration(scope.upper, query);
}
exports.default = create_testing_library_rule_1.createTestingLibraryRule({
    name: exports.RULE_NAME,
    meta: {
        type: 'suggestion',
        docs: {
            description: 'Suggest using find* instead of waitFor to wait for elements',
            category: 'Best Practices',
            recommended: 'warn',
        },
        messages: {
            preferFindBy: 'Prefer {{queryVariant}}{{queryMethod}} method over using await {{fullQuery}}',
        },
        fixable: 'code',
        schema: [],
    },
    defaultOptions: [],
    create(context, _, helpers) {
        const sourceCode = context.getSourceCode();
        function reportInvalidUsage(node, { queryVariant, queryMethod, fix, }) {
            context.report({
                node,
                messageId: 'preferFindBy',
                data: {
                    queryVariant,
                    queryMethod,
                    fullQuery: sourceCode.getText(node),
                },
                fix,
            });
        }
        return {
            'AwaitExpression > CallExpression'(node) {
                if (!experimental_utils_1.ASTUtils.isIdentifier(node.callee) ||
                    !exports.WAIT_METHODS.includes(node.callee.name)) {
                    return;
                }
                const argument = node.arguments[0];
                if (!node_utils_1.isArrowFunctionExpression(argument)) {
                    return;
                }
                if (!node_utils_1.isCallExpression(argument.body)) {
                    return;
                }
                if (node_utils_1.isMemberExpression(argument.body.callee) &&
                    experimental_utils_1.ASTUtils.isIdentifier(argument.body.callee.property) &&
                    experimental_utils_1.ASTUtils.isIdentifier(argument.body.callee.object) &&
                    helpers.isSyncQuery(argument.body.callee.property)) {
                    const fullQueryMethod = argument.body.callee.property.name;
                    const caller = argument.body.callee.object.name;
                    const queryVariant = getFindByQueryVariant(fullQueryMethod);
                    const callArguments = argument.body.arguments;
                    const queryMethod = fullQueryMethod.split('By')[1];
                    reportInvalidUsage(node, {
                        queryMethod,
                        queryVariant,
                        fix(fixer) {
                            const property = argument.body
                                .callee.property;
                            if (helpers.isCustomQuery(property)) {
                                return;
                            }
                            const newCode = `${caller}.${queryVariant}${queryMethod}(${callArguments
                                .map((node) => sourceCode.getText(node))
                                .join(', ')})`;
                            return fixer.replaceText(node, newCode);
                        },
                    });
                    return;
                }
                if (!experimental_utils_1.ASTUtils.isIdentifier(argument.body.callee) ||
                    !helpers.isSyncQuery(argument.body.callee)) {
                    return;
                }
                const fullQueryMethod = argument.body.callee.name;
                const queryMethod = fullQueryMethod.split('By')[1];
                const queryVariant = getFindByQueryVariant(fullQueryMethod);
                const callArguments = argument.body.arguments;
                reportInvalidUsage(node, {
                    queryMethod,
                    queryVariant,
                    fix(fixer) {
                        if (helpers.isCustomQuery(argument.body
                            .callee)) {
                            return;
                        }
                        const findByMethod = `${queryVariant}${queryMethod}`;
                        const allFixes = [];
                        const newCode = `${findByMethod}(${callArguments
                            .map((node) => sourceCode.getText(node))
                            .join(', ')})`;
                        allFixes.push(fixer.replaceText(node, newCode));
                        const definition = findRenderDefinitionDeclaration(context.getScope(), fullQueryMethod);
                        if (!definition) {
                            return allFixes;
                        }
                        if (node_utils_1.isObjectPattern(definition.parent.parent)) {
                            const allVariableDeclarations = definition.parent.parent;
                            if (allVariableDeclarations.properties.some((p) => node_utils_1.isProperty(p) &&
                                experimental_utils_1.ASTUtils.isIdentifier(p.key) &&
                                p.key.name === findByMethod)) {
                                return allFixes;
                            }
                            const textDestructuring = sourceCode.getText(allVariableDeclarations);
                            const text = textDestructuring.substring(0, textDestructuring.length - 2) +
                                `, ${findByMethod} }`;
                            allFixes.push(fixer.replaceText(allVariableDeclarations, text));
                        }
                        return allFixes;
                    },
                });
            },
        };
    },
});
